import { createSlice, PayloadAction } from '@reduxjs/toolkit';
import type { RootState } from './store';
import { VisualizationSettingsType } from '../../vis/common';
import { isEqual } from 'lodash-es';

export type VisStateSettings = VisualizationSettingsType[];
export type VisState = {
  activeVisualizationIndex: number;
  openVisualizationArray: VisStateSettings;
};

export const initialState: VisState = {
  activeVisualizationIndex: -1,
  openVisualizationArray: [],
};

export const visualizationSlice = createSlice({
  name: 'visualization',
  initialState,
  reducers: {
    removeVisualization: (state, action: PayloadAction<number | undefined>) => {
      let index = action.payload || state.activeVisualizationIndex;
      state.openVisualizationArray.splice(index, 1);
      if (state.activeVisualizationIndex === index) {
        if (state.openVisualizationArray.length === 0) state.activeVisualizationIndex = -1;
        else {
          state.activeVisualizationIndex = Math.max(state.openVisualizationArray.length - 1, 0);
        }
      }
    },
    setActiveVisualization: (state, action: PayloadAction<number>) => {
      state.activeVisualizationIndex = action.payload;
    },
    setVisualizationState: (state, action: PayloadAction<VisState>) => {
      if (action.payload.activeVisualizationIndex !== undefined && !isEqual(action.payload, state)) {
        state.openVisualizationArray = action.payload.openVisualizationArray || [];
        state.activeVisualizationIndex = action.payload.activeVisualizationIndex;
      }
    },
    updateVisualization: (state, action: PayloadAction<{ id: number; settings: VisualizationSettingsType }>) => {
      const { id, settings } = action.payload;
      state.openVisualizationArray[id] = settings;
    },
    addVisualization: (state, action: PayloadAction<VisualizationSettingsType>) => {
      state.openVisualizationArray.push(action.payload);
      state.activeVisualizationIndex = state.openVisualizationArray.length - 1;
    },
    updateActiveVisualization: (state, action: PayloadAction<VisualizationSettingsType>) => {
      if (state.activeVisualizationIndex === -1) {
        return;
      }

      state.openVisualizationArray[state.activeVisualizationIndex] = action.payload;
    },
    updateActiveVisualizationAttributes: (state, action: PayloadAction<Record<string, any>>) => {
      if (state.activeVisualizationIndex === -1) {
        return;
      }

      state.openVisualizationArray[state.activeVisualizationIndex] = {
        ...state.openVisualizationArray[state.activeVisualizationIndex],
        ...action.payload,
      };
    },
    reorderVisState: (state, action: PayloadAction<VisStateSettings>) => {
      state.openVisualizationArray = action.payload;
    },
  },
});

export const {
  removeVisualization,
  setActiveVisualization,
  updateActiveVisualization,
  setVisualizationState,
  updateVisualization,
  reorderVisState,
  addVisualization,
  updateActiveVisualizationAttributes,
} = visualizationSlice.actions;

export const visualizationState = (state: RootState) => state.visualize;
export const visualizationAllSettings = (state: RootState) => state.visualize.openVisualizationArray;
export const visualizationActive = (state: RootState) =>
  state.visualize.activeVisualizationIndex !== -1
    ? state.visualize.openVisualizationArray?.[state.visualize.activeVisualizationIndex]
    : undefined;
export default visualizationSlice.reducer;
