import React, { useState, useEffect, useRef } from 'react';
import { Button, DropdownContainer, DropdownItem, DropdownItemContainer, DropdownTrigger } from '../../components';
import { Tooltip, TooltipContent, TooltipProvider, TooltipTrigger } from '../../components/tooltip';
import { ControlContainer } from '../../components/controls';
import { Tab } from '../../components/tabs';
import { useAppDispatch, useVisualization } from '../../data-access';
import { addVisualization, removeVisualization, reorderVisState, setActiveVisualization } from '../../data-access/store/visualizationSlice';
import { Visualizations } from './VisualizationPanel';
import { Reorder, AnimatePresence } from 'framer-motion';

export default function VisualizationTabBar(props: { fullSize: () => void }) {
  const dispatch = useAppDispatch();
  const { activeVisualizationIndex, openVisualizationArray } = useVisualization();
  const tabsRef = useRef<HTMLDivElement | null>(null);
  const [open, setOpen] = useState(false);
  const [showMore, setShowMore] = useState(false);

  const onSelect = async (id: number) => {
    dispatch(setActiveVisualization(id));
  };

  const onDelete = (id: number) => {
    dispatch(removeVisualization(id));
  };

  const handleReorder = (newOrder: any[]) => {
    dispatch(reorderVisState(newOrder));
  };

  const updateOverflowState = () => {
    if (tabsRef.current) {
      const { scrollWidth, clientWidth, scrollLeft } = tabsRef.current;
      setShowMore(scrollWidth > clientWidth);
    }
  };

  useEffect(() => {
    updateOverflowState();
    window.addEventListener('resize', updateOverflowState);
    return () => {
      window.removeEventListener('resize', updateOverflowState);
    };
  }, [openVisualizationArray]);

  useEffect(() => {
    updateOverflowState();
  }, [openVisualizationArray]);

  return (
    <div className="sticky shrink-0 top-0 flex items-center justify-between h-7 bg-secondary-100 border-b border-secondary-200 max-w-full">
      <div className="flex items-center shrink-0 px-0.5">
        <TooltipProvider delayDuration={0}>
          <Tooltip>
            <TooltipTrigger>
              <DropdownContainer open={open} onOpenChange={setOpen}>
                <DropdownTrigger onClick={() => setOpen((v) => !v)}>
                  <Button
                    as={'a'}
                    variantType="secondary"
                    variant="ghost"
                    size="xs"
                    iconComponent="icon-[ic--baseline-add]"
                    onClick={() => {}}
                  />
                </DropdownTrigger>
                <DropdownItemContainer>
                  {Object.keys(Visualizations).map((name, i) => (
                    <DropdownItem
                      value={name}
                      key={name}
                      className=""
                      onClick={async (e) => {
                        const component = await Visualizations[name]();
                        dispatch(addVisualization({ ...component.default.settings, name: name, id: name }));
                        setOpen(false);
                      }}
                    ></DropdownItem>
                  ))}
                </DropdownItemContainer>
              </DropdownContainer>
            </TooltipTrigger>
            {openVisualizationArray.length !== 0 && (
              <TooltipContent>
                <p>Add visualization</p>
              </TooltipContent>
            )}
          </Tooltip>
        </TooltipProvider>
      </div>

      {openVisualizationArray.length === 0 && (
        <div className="flex items-center">
          <h1 className="text-xs font-semibold text-secondary-600 pr-2">Select visualization</h1>
        </div>
      )}

      <div className="relative flex items-center h-full flex-1 overflow-hidden">
        <Reorder.Group
          as="div"
          axis="x"
          values={openVisualizationArray}
          onReorder={handleReorder}
          className="flex flex-1 items-center h-full divide-x divide-secondary-200 border-x border-secondary-200 overflow-x-auto"
          style={{
            overflowX: 'auto',
            WebkitOverflowScrolling: 'touch',
            msOverflowStyle: 'none',
            scrollbarWidth: 'none',
          }}
          ref={tabsRef}
        >
          <AnimatePresence>
            {openVisualizationArray.map((vis, i) => (
              <Reorder.Item
                key={i}
                value={vis}
                className="flex flex-grow items-center h-full"
                initial={{ opacity: 0 }}
                animate={{ opacity: 1 }}
                exit={{ opacity: 0, y: 20, transition: { duration: 0.3 } }}
              >
                <Tab activeTab={activeVisualizationIndex === i} text={vis.name} onClick={() => onSelect(i)}>
                  <Button
                    variantType="secondary"
                    variant="ghost"
                    rounded
                    size="2xs"
                    iconComponent="icon-[ic--baseline-close]"
                    onClick={(e) => {
                      e.stopPropagation();
                      onDelete(i);
                    }}
                  />
                </Tab>
              </Reorder.Item>
            ))}
          </AnimatePresence>
        </Reorder.Group>

        {showMore && (
          <div className="h-full flex items-center justify-center ml-auto">
            <div className="w-8 h-full flex items-center justify-center text-secondary-500 bg-secondary-200">
              <span className="text-md">…</span>
            </div>
          </div>
        )}
      </div>

      <div className="px-0.5 ml-auto items-center flex">
        <ControlContainer>
          <TooltipProvider delayDuration={0}>
            <Tooltip>
              <TooltipTrigger asChild>
                <Button
                  variantType="secondary"
                  variant="ghost"
                  size="xs"
                  iconComponent="icon-[ic--baseline-fullscreen]"
                  onClick={props.fullSize}
                />
              </TooltipTrigger>
              <TooltipContent>
                <p>Full screen</p>
              </TooltipContent>
            </Tooltip>
            <Tooltip>
              <TooltipTrigger asChild>
                <Button
                  variantType="secondary"
                  variant="ghost"
                  size="xs"
                  iconComponent="icon-[ic--baseline-camera-alt]"
                  onClick={() => {}}
                />
              </TooltipTrigger>
              <TooltipContent>
                <p>Capture visualization</p>
              </TooltipContent>
            </Tooltip>
          </TooltipProvider>
        </ControlContainer>
      </div>
    </div>
  );
}
